use std::sync::Arc;

use cozo::DbInstance;
use tracing::debug;

use crate::{state::{types::ElementId, queries}, comm::messages::MessageContent};

use super::{State, types::{ElementContent, Element}};



pub struct ApiState {
    state: Arc<State>,
}

impl ApiState {
    pub fn new(state: Arc<State>) -> Self {
        ApiState { state: state }
    }

    pub fn create_element(&self, content: &ElementContent) -> anyhow::Result<ElementId> {
        let id = ElementId::new();
        queries::elements::add(self.db(), &id, &content, None, true)?;
        debug!("Added element {{{}}}", &id.to_string());

        self.state.send_to_peers(MessageContent::CreateElement { id: id.clone(), content: content.clone() });
        Ok(id)
    }

    pub fn write_element_content(&self, id: &ElementId, content: &ElementContent) -> anyhow::Result<()> {
        queries::elements::set_content(self.db(), id, content)?;
        queries::elements::set_local_changes(self.db(), id, true)?;
        debug!("Wrote element content {{{}}}", &id.to_string());

        Ok(())
    }

    pub fn remove_element(&self, id: &ElementId) -> anyhow::Result<()> {
        let res = self.state.remove_element(id);
        debug!("Removed element {{{}}}", &id.to_string());

        res
    }

    pub fn get_element(&self, id: &ElementId) -> anyhow::Result<Element> {
        self.state.get_element(id)
    }


    fn db(&self) -> &DbInstance {
        &self.state.db
    }
}

#[cfg(test)]
mod tests {
    use super::ApiState;
    use crate::state::{types::ElementContent, State};
    
    #[tokio::test]
    #[serial_test::serial]
    async fn test_element_create() {
        tracing_subscriber::fmt().pretty().init();
        let state = ApiState::new(State::new().await.unwrap());
        let id = state.create_element(&ElementContent::Text("Test-text".to_string())).unwrap();
        let el = state.get_element(&id).unwrap();
        assert_eq!(
            ElementContent::Text("Test-text".to_string()),
            el.content().to_owned()
        )
    }

    #[tokio::test]
    #[serial_test::serial]
    async fn test_element_write() {
        tracing_subscriber::fmt().pretty().init();
        let state = ApiState::new(State::new().await.unwrap());
        let id = state.create_element(&ElementContent::Text("Test-text".to_string())).unwrap();
        state.write_element_content(&id,&ElementContent::Text("Test-text 2".to_string())).unwrap();
        let el = state.get_element(&id).unwrap();
        assert_eq!(
            ElementContent::Text("Test-text 2".to_string()),
            el.content().to_owned()
        )
    }
}
